/*
 * Copyright (c) 2003-2006 Erez Zadok
 * Copyright (c) 2003-2006 Stony Brook University
 * Copyright (c) 2003-2006 The Research Foundation of State University of New York
 * Copyright (c) 2004-2006 Nikolai Joukov
 * Copyright (c) 2003-2005 Akshat Aranya
 * Copyright (c) 2006-2006 Gopala Surya
 */

#include "tracereader.h"
#include "util.h"
#include <openssl/evp.h>

static char iv[EVP_MAX_IV_LENGTH] = { 0 };
static EVP_CIPHER_CTX ctx;
static const EVP_CIPHER *evp_type;


int get_decrypted_value(int anon_type, void *invalue, int inlen, void **outvalue, int *outl)
{
	int outlen;
	int err = ERR_OK;
	char *outbuf = NULL;
	char *ptr;
	evp_type = EVP_bf_cbc();
	*outvalue = NULL;
	EVP_CIPHER_CTX_init(&ctx);
	ptr = outbuf = malloc(inlen + EVP_CIPHER_key_length(evp_type));
	if (!outbuf) {
		err = ERR_NOMEM;
		goto out;
	}
	if (!EVP_DecryptInit(&ctx, evp_type, (unsigned char *)anon_keys[anon_type].key, (unsigned char *)iv)) {
		err = ERR_INVAL;
		goto out;
	}
	if (!EVP_DecryptUpdate(&ctx, (unsigned char *)ptr, &outlen, invalue, inlen)) {
		err = ERR_IO;
		goto out;
	}
	ptr += outlen;
	if (!EVP_DecryptFinal(&ctx, (unsigned char *)ptr, &outlen)) {
		err = ERR_IO;
		goto out;
	}
	ptr += outlen;
	*outvalue = outbuf;
	*outl = ptr - outbuf;
out:
	EVP_CIPHER_CTX_cleanup(&ctx);
	if (err != ERR_OK && outbuf)
		free(outbuf);
	return err;
}

void release_decrypted_value(void *dvalue)
{
	free(dvalue);
}

