/*
 * Copyright (c) 2003-2006 Erez Zadok
 * Copyright (c) 2003-2006 Stony Brook University
 * Copyright (c) 2003-2006 The Research Foundation of State University of New York
 * Copyright (c) 2004-2006 Nikolai Joukov
 * Copyright (c) 2003-2005 Akshat Aranya
 * Copyright (c) 2006-2006 Gopala Surya
 */

#include "idriver.h"
#include <openssl/evp.h>
#include "common.h"
#include <string.h>
#include <zlib.h>

static z_stream stream;
static int z_ret;
static int z_eof;

#define BUF_SIZE 4096

static char buf[4096];

static int zdriver_init(struct parser *p, void *params, struct idriver *next)
{
	if (!next) {
		snprintf(p->err, ERR_SIZE, "uncompress driver requires a next driver");
		return ERR_INVAL;
	}
	uncompress_idriver->next_driver = next;

	stream.zalloc = (alloc_func)0;
	stream.zfree = (free_func)0;
	stream.opaque = (voidpf)0;
	stream.next_in = (Byte *)buf;
	stream.avail_in = 0;

	stream.next_out = 0;
	stream.avail_out = 0;
	z_eof = 0;
	z_ret = inflateInit(&stream);
	return (z_ret == Z_OK) ? ERR_OK : ERR_NOMEM;

}

static int zdriver_read(struct parser *p, char *buffer, int len)
{
	Byte *next_out;
	if (z_ret == Z_DATA_ERROR || z_ret == Z_ERRNO) return -ERR_IO;
	if (z_ret == Z_STREAM_END) return 0;

	next_out = (Byte *)buffer;
	stream.next_out = (Bytef*)buffer;
	stream.avail_out = len;

	while (stream.avail_out != 0) {
		if (stream.avail_in == 0 && !z_eof) {
			stream.avail_in = uncompress_idriver->next_driver->ops->read(p, buf, BUF_SIZE);
			if (stream.avail_in < 0)
				return stream.avail_in;
			else if (stream.avail_in == 0)
				z_eof = 1;
			stream.next_in = (Byte *)buf;
		}
		z_ret = inflate(&stream, Z_NO_FLUSH);
		if (z_ret == Z_STREAM_END) z_ret = Z_OK;
		if (z_ret != Z_OK || z_eof) break;
	}
	return (z_ret != Z_OK) ? -ERR_IO : (len - stream.avail_out);

}

static void zdriver_release()
{
	inflateEnd(&stream);
}

struct input_ops zdriver_ops  = {
	init: zdriver_init,
	read: zdriver_read,
	release: zdriver_release
};

struct idriver zdriver = {
	ops : &zdriver_ops
};

struct idriver *uncompress_idriver = &zdriver;
