/*
 * Copyright (c) 2003-2006 Erez Zadok
 * Copyright (c) 2003-2006 Stony Brook University
 * Copyright (c) 2003-2006 The Research Foundation of State University of New York
 * Copyright (c) 2004-2006 Nikolai Joukov
 * Copyright (c) 2003-2005 Akshat Aranya
 * Copyright (c) 2006-2006 Gopala Surya
 */

#include <string.h>
#include "pidhash.h"
#include <stdlib.h>
#include <stdio.h>

struct pidhash *pidhash_create(int buckets)
{
	int reqd = sizeof(struct pidhash) + buckets * sizeof(struct pidhash_entry);
	int i;
	struct pidhash *tmp = malloc(reqd);

	if (tmp) {
            memset(tmp,0,sizeof(reqd));
	    for (i = 0; i < buckets; i++) {
		tmp->buckets[i].next = tmp->buckets[i].prev = &tmp->buckets[i];
	    }
	    tmp->size = buckets;
	}
	return tmp;
}

static void destroy_pidhash_entry(struct pidhash_entry *entry)
{
    free(entry->pname);
    free(entry);
}



void pidhash_destroy(struct pidhash *hash)
{
	int i;
	for (i = 0; i < hash->size; i++)
	{
		struct pidhash_entry *tmp = hash->buckets[i].next;
		struct pidhash_entry *tmp2 = NULL;
		while (tmp != &hash->buckets[i]) {
		    tmp2 = tmp->next;
		    destroy_pidhash_entry(tmp);
		    tmp = tmp2;
		}
	}
}

static pid_t hash(struct pidhash *hash, pid_t pid)
{
	int hval = pid % hash->size;
	return (hval < 0) ? -hval : hval;
}

struct pidhash_entry *pidhash_lookup(struct pidhash *hasht, pid_t pid)
{
	int bucket = hash(hasht, pid);
	struct pidhash_entry *tmp = hasht->buckets[bucket].next;
	while(tmp != &hasht->buckets[bucket]) {
		if (tmp->pid == pid)
			break;
		tmp = tmp->next;
	}
	if (tmp != &hasht->buckets[bucket])
	    return tmp;
	else
	    return NULL;
}


/* This replaces an old entry from the hash table */
struct pidhash_entry *pidhash_add_entry(struct pidhash *hasht, pid_t pid, char *name, int len, struct pidhash_entry *current)
{

//	int bucket = hash(hasht, pid);
	struct pidhash_entry *tmp;
	struct pidhash_entry *new = malloc(sizeof(struct pidhash_entry));

	memset(new,0,sizeof(struct pidhash_entry));
	if (new) {
	    new->pid = pid;
	    new->pname = malloc(len);
	    if (new->pname) {
		new->pname = memcpy(new->pname, name, len);
		new->len = len;
		if (current) {
			new->next = current->next;
			new->prev = current->prev;
			if(new->next)
			  new->next->prev = new;
			if(new->prev)
			  new->prev->next = new;
			destroy_pidhash_entry(current);
		} else {
			tmp = &(hasht->buckets[hash(hasht, pid)]);
			new->next = tmp->next;
			new->prev = tmp;
			tmp->next = new;
			if(new->next)
			  new->next->prev = new;
		}
	    } else {
		free(new);
		new = NULL;
	    }
	}
	return new;
}
