/*
 * Copyright (c) 2003-2006 Erez Zadok
 * Copyright (c) 2003-2006 Stony Brook University
 * Copyright (c) 2003-2006 The Research Foundation of State University of New York
 * Copyright (c) 2004-2006 Nikolai Joukov
 * Copyright (c) 2003-2005 Akshat Aranya
 * Copyright (c) 2006-2006 Gopala Surya
 *
 * For specific licensing information, see the COPYING file distributed with
 * this package, or get one from ftp://ftp.filesystems.org/pub/fist/COPYING.
 *
 * This Copyright notice must be kept intact and distributed with all
 * fistgen sources INCLUDING sources generated by fistgen.
 */

/*
 * This header contains utility functions for writing
 * objects in a stream format into the trace.
 */

#ifndef __SERIALIZER_H
#define __SERIALIZER_H
#ifdef KWRAPPERS
#include "kwrappers.h"
#endif
#define DATA_SIZE (PAGE_SIZE - sizeof(int))


/* Sequence Numbers for binding PRE/POST ops to the same context in the user-level parser */
static atomic_t sequence_number;

/* initial location to write to in the buffer, basically excluding the operation, sequence numbers, and length fields. */
#define LEN_LOC (sizeof(op_t /* op */))
#define SEQNUM_LOC (sizeof(op_t) + sizeof(msg_len_t))
#define INIT_LOC (LEN_LOC + sizeof(msg_len_t) + sizeof(msg_seqnum_t /* sequence number */))

struct buffer {
    char *ptr;
    char data[DATA_SIZE];
};

static inline struct buffer *get_buffer(void)
{
	/* XXX:  __get_free_page should be used.  KMALLOC is for small stuff < PAGE_SIZE */
    struct buffer *buf = KMALLOC(sizeof(struct buffer), GFP_KERNEL);
    if (buf)
	buf->ptr = buf->data + INIT_LOC;
    return buf;
}

static inline struct buffer *get_buffer_nofs(void)
{
    struct buffer *buf = KMALLOC(sizeof(struct buffer), GFP_NOFS);
    if (buf)
	buf->ptr = buf->data + INIT_LOC;
    return buf;
}

#define release_buffer(buf) (KFREE(buf))
#define reset_buffer(buf) ((buf)->ptr = (buf)->data + INIT_LOC)

#define remaining(buf) ((buf)->data + DATA_SIZE - (buf)->ptr)

#define add_tv(buf, type, value) \
    do { \
	if (remaining(buf) >= sizeof(field_t) + sizeof(value)) { \
	        field_t tmp = (type); \
	        memcpy((buf)->ptr, &tmp, sizeof(field_t)); \
	        (buf)->ptr += sizeof(field_t); \
	        memcpy((buf)->ptr, &value, sizeof(value)); \
        	(buf)->ptr += sizeof(value); \
	} \
    } while (0);

#define add_tv_ptr(buf, type, len, pvalue) \
    do { \
	if (remaining(buf) >= sizeof(field_t) + len) { \
	        field_t tmp = (type); \
        	memcpy((buf)->ptr, &tmp, sizeof(field_t)); \
	        (buf)->ptr += sizeof(field_t); \
	        memcpy((buf)->ptr, pvalue, len); \
		(buf)->ptr += len; \
	} \
    } while (0);


#define add_tlv(buf, type, length, value) \
    do { \
	if (remaining(buf) >= sizeof(field_t) + sizeof(field_len_t) + length) { \
	        field_t tmp = type; \
	        field_len_t len = length; \
	        tmp |= FIELD_LENGTH_MASK; \
	        memcpy((buf)->ptr, &tmp, sizeof(field_t)); \
	        (buf)->ptr += sizeof(field_t); \
		memcpy((buf)->ptr, &len, sizeof(len)); \
        	(buf)->ptr += sizeof(field_len_t); \
	        memcpy((buf)->ptr, value, length); \
        	(buf)->ptr += length; \
	} \
    } while (0);

// You *must* append the value at the end on your own
// if you intend to use this function.
// otherwise the trace will not be well formed
#define add_tl(buf, type, length) \
    do { \
	if (remaining(buf) >= sizeof(field_t) + sizeof(field_len_t)) { \
	        field_t tmp = type; \
	        field_len_t len = length; \
	        tmp |= FIELD_LENGTH_MASK; \
	        memcpy((buf)->ptr, &tmp, sizeof(field_t)); \
	        (buf)->ptr += sizeof(field_t); \
		memcpy((buf)->ptr, &len, sizeof(len)); \
        	(buf)->ptr += sizeof(field_len_t); \
	} \
    } while (0);


#define add_ret(buf, ret) add_tv((buf), ARG_RET, (ret))

static inline void end_trace_message(struct buffer *buf)
{
    msg_len_t len = buf->ptr - buf->data - INIT_LOC;
    memcpy(buf->data + LEN_LOC, &len, sizeof(msg_len_t));
}


#endif
