/*
 * Copyright (c) 2003-2006 Erez Zadok
 * Copyright (c) 2003-2006 Stony Brook University
 * Copyright (c) 2003-2006 The Research Foundation of State University of New York
 * Copyright (c) 2004-2006 Nikolai Joukov
 * Copyright (c) 2003-2005 Akshat Aranya
 * Copyright (c) 2006-2006 Gopala Surya
 *
 * For specific licensing information, see the COPYING file distributed with
 * this package, or get one from ftp://ftp.filesystems.org/pub/fist/COPYING.
 *
 * This Copyright notice must be kept intact and distributed with all
 * fistgen sources INCLUDING sources generated by fistgen.
 */
#include <errno.h>
#include <time.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include "common.h"
#include <stdio.h>


void usage()
{
    fprintf(stderr, "Usage: gostop <mount point> <action> <id>\n");
    fprintf(stderr, "  actions are:\n");
    fprintf(stderr, "  go:     start trace\n");
    fprintf(stderr, "  stop:   stop trace\n");
    fprintf(stderr, "  da:     delete assembly drivers\n");
    fprintf(stderr, "  do:     delete output drivers\n");
    fprintf(stderr, "  dr:     delete rule\n");
    fprintf(stderr, "  d:      delete all (teardown)\n\n");
    fprintf(stderr, "  id is the destination id of the trace\n\n");
    exit(1);
}

int do_exit(const char *s)
{
    perror(s);
    exit(errno);
}

int main(int argc, char *argv[])
{

    int ret;
    int fd;
    int op;

    if (argc < 4)
	usage();
    fd = open(argv[1],  O_RDONLY);
    if (fd < 0)
	do_exit("open");

    if (strcmp(argv[2], "go") == 0)
	op = TRACEFS_IOCTL_START_TRACE;
    else if (strcmp(argv[2], "stop") == 0)
	op = TRACEFS_IOCTL_STOP_TRACE;
    else if (strcmp(argv[2], "da") == 0)
	op = TRACEFS_IOCTL_DEL_ASSEMBLY_DRIVERS;
    else if (strcmp(argv[2], "do") == 0)
	op = TRACEFS_IOCTL_DEL_OUTPUT_DRIVERS;
    else if (strcmp(argv[2], "dr") == 0)
	op = TRACEFS_IOCTL_DEL_RULE;
    else if (strcmp(argv[2], "d") == 0)
	op = TRACEFS_IOCTL_TEARDOWN;

    else
	usage();

    ret = ioctl(fd, op, atoi(argv[3]));

    if (ret < 0)
	do_exit("ioctl");

    return 0;
}
